// last modified: 2009-02-09

#include <SDL/SDL.h>                                                                                // main SDL-header
#include <SDL/SDL_ttf.h>                                                                            // TTF to render text to surfaces
#include <stdio.h>                                                                                  // standard functions
#include <stdlib.h>                                                                                 // - " -
#include <string.h>                                                                                 // used to copy strings, define string length, error messages etc
#include <time.h>                                                                                   // used to get current time to initialize random number generator
#include <errno.h>                                                                                  // get error numbers to print error messages
#include <ctype.h>                                                                                  // contains functions to ask for type of a character



#define VERSION 0.2                                                                                 // version of the program
#define SUBVERSION ' '                                                                              // subversion for small changes
#define CHARVERSION "0.2"                                                                           // version of the program in characters

#define ERROR_SDL 0                                                                                 // constant to show that a SDL-error occured
#define ERROR_TTF 1                                                                                 // constant to show that a TTF-error occured 
#define ERROR_FILE 2                                                                                // constant to show that an error occured opening a file

#define MAXLENGTH 5000                                                                              // maximum number of blocks

#define MAXSCORES 5                                                                                 // maximum number of scores in the highscore list

#define UP 0                                                                                        // constant for moving up
#define RIGHT 1                                                                                     // constant for moving right
#define DOWN 2                                                                                      // constant for moving down
#define LEFT 3                                                                                      // constant for moving left



typedef struct node                                                                                 // structure to store informations of each block
{
  int direction;                                                                                    // direction the block is moving
  float x, y;                                                                                       // x and y coordinates of the block (float to get exact results)
  struct node *prev;                                                                                // pointer to previous block
  struct node *next;                                                                                // pointer to next block
} block_info;


typedef struct                                                                                      // structure to store informations of the fruit
{
  float x, y;                                                                                       // x and y coordinates of the fruit
} fruit_struct;


typedef struct                                                                                      // structure to get highscores from a file
{
  unsigned int score;                                                                               // reached score
  char name[16];                                                                                    // name of the player
} highscoreinfo;


typedef struct                                                                                      // structure to blit the highscorelist
{
  SDL_Surface *pos;                                                                                 // ranking of the score
  SDL_Surface *name;                                                                                // name of the player
  SDL_Surface *score;                                                                               // reached score
} scoresurfaces;


typedef struct                                                                                      // structure to split score-informations into single strings
{
  char pos[3];                                                                                      // ranking 
  char name[16];                                                                                    // name of the player
  char score[9];                                                                                    // reached score
} charscores;


typedef struct                                                                                      // structure to save resolution modes
{
  int w, h;                                                                                         // width and height of the window
} resmodes;



block_info *snake, *oldpos;                                                                         // headpointers for the lists
SDL_Surface *screen;                                                                                // surface that represents the screen
fruit_struct fruit;                                                                                 // the fruit
int length;                                                                                         // actual length of the snake
unsigned int frames = 0;                                                                            // stores number of frames since last check
float level = 1.0, fps = 0.0, speed = 0.0;                                                          // the difficulty level, frames per second and the speed



void removelogerror ();                                                                             // deletes the error log, to create a new one
void logerror (int type);                                                                           // write error messages into a file
void load_config (int *w, int *h, int *fullscreen, int readonly);                                   // load configuration from a file
int menu ();                                                                                        // draw menu and return decision
void game ();                                                                                       // starts the game
void settings (int w, int h, int fullscreen);                                                       // change settings
void main_loop ();                                                                                  // main game loop
void parameters (int *w, int *h, int *fullscreen, int argc, char *argv[]);                          // checks command line parameters
int parameter_width (char *flag, char *par, int *w);                                                // check for "-w" flag to define the window width
int parameter_height (char *flag, char *par, int *h);                                               // check for "-h" flag to define the window height
void parameter_help (char *flag);                                                                   // check for "-help" flag to display informations in the terminal
int parameter_fullscreen (char *flag, int *fullscreen);                                             // check for "--full" flag to start game in fullscreen mode
void draw_snake ();                                                                                 // draws the snake on the surface
void draw_fruit ();                                                                                 // draws the fruit on the surface
void new_fruit ();                                                                                  // get the position of a new fruit
int onsnake (float x, float y);                                                                     // check if fruit is on the snake
void check_input (SDL_Event event, int *run);                                                       // checks which key was pressed using SDL_Event
void check_border_score (int *run, unsigned int *score);                                            // checks if the snake hit a border or ate a fruit
void move_snake ();                                                                                 // the snake gets a new position
void display_help ();                                                                               // displays informations in the terminal
void pause (int *run);                                                                              // pause the game
void create_blocks (int nr);                                                                        // appends a new block to the snake
void draw_end (unsigned int score);                                                                 // draws the score and "game over" on the screen
void save_locations ();                                                                             // saves the location and direction of the blocks
int bit_self ();                                                                                    // checks if snake bit itself
void pause_screen ();                                                                               // draws a pause-screen
void rendering ();                                                                                  // render background, fruit and snake
int start_SDL (int w, int h, int fullscreen);                                                       // initializes SDL and TTF and sets video mode
void calculate_speed (int *lasttime);                                                               // calculates speed
void check_events (int *run);                                                                       // check for SDL events
void starting_snake_fruit ();                                                                       // creates the snake and fruit for the beginning
void check_hit_border (int bitself, unsigned int score, int *run);                                  // check if snake hit the border
void check_ate_fruit (unsigned int *score);                                                         // check if the snake ate a fruit
void draw_blockwithframe (int x, int y, int w, int h);                                              // draws a black block with a white frame
void highscore (unsigned int reachedscore);                                                         // checks if the score is in the top-5 and enters it in the list
void entryscore (int pos, int nr, highscoreinfo newscore, highscoreinfo list[]);                    // enters score into the highscore list
void draw_highscorelist (int newpos);                                                               // draws the highscore list on the screen
void draw_title (char text[]);                                                                      // draws a title on the screen
void scorecpy (highscoreinfo *dest, highscoreinfo src);                                             // copies score informations from one structure to another
void getname (char name[]);                                                                         // get name using SDL-events
char sdltoascii (SDL_Event event);                                                                  // converts a SDL event to an ASCII character
void draw_input (char *name);                                                                       // draw the typed characters of "getname"
void draw_nametitle ();                                                                             //void load_config (int *w, int *h, int *fullscreen); asks for a name
int yesorno (char text[]);                                                                          // asks the player a question
void deletesnake ();                                                                                // free allocated memory



int main (int argc, char *argv[])                                                                   // the program accepts command line parameters
{

  int  w, h, fullscreen,                                                                            // width and hight of the window; say fullscreen or not
       action, quit;                                                                                // decision form menu, asks player if really wants to quit
  char charsure[] = "Sure?";                                                                        // text to ask player if he really wants to quit

  srand (time (NULL));                                                                              // initialize random number generator with local time

  load_config (&w, &h, &fullscreen, 1);                                                             // get settings from the configuration file

  removelogerror ();

  if (argc > 1)                                                                                     // check if there are extra parameters
    parameters (&w, &h, &fullscreen, argc, argv);                                                   // check command line parameters


  if ((start_SDL (w, h, fullscreen)))                                                               // initializes SDL and TTF and sets video mode; check if all went right
    return 1;                                                                                       // stop program if something went wrong


  do
  {

    action = menu ();                                                                               // get decision from the menu
    
    quit = 0;                                                                                       // don't quit game
    
    switch (action)                                                                                 // switch the action
    {
     
      case 0: game ();                                                                              // start game
              break;
              
      case 1: draw_highscorelist (-1);                                                              // show highscorelist
              break;
              
      case 2: settings (w, h, fullscreen);                                                          // change settings
              load_config (&w, &h, &fullscreen, 1);                                                 // reload settings from the configuration file
              break;                       
              
      case 3: quit = yesorno (charsure);                                                            // asks if player really wants to quit the game
              break;
    
    }
  
  } while (!quit);                                                                                  // play until player really wants to quit


  return 0;

}



void load_config (int *w, int *h, int *fullscreen, int readonly)                                    // get settings from the configuration file
{

  FILE *config;                                                                                     // file pointer to the configuration file

  if ((config = fopen (".EatTheBlocks.conf", "r +")) && readonly)                                   // try to open file for reading                 
  {
   
    fscanf (config, "%d * %d\nfullscreen: %d", w, h, fullscreen);                                   // get the values from the file
    
    fclose (config);                                                                                // close the file
    
  }  
  
    else if ((config = fopen (".EatTheBlocks.conf", "w")) && !readonly)                             // change the cnfig
    {
    
      fprintf (config, "%d * %d\nfullscreen: %d", *w, *h, *fullscreen);                             // write the informations in the file
      
      fclose (config);                                                                              // close the file
    
    }
  
      else if ((config = fopen (".EatTheBlocks.conf", "w")))                                        // no config available, a new one is created
      {  
    
        *w = 800;                                                                                   // set standard settings
        *h = 600;
        *fullscreen = 0;
      
        fprintf (config, "%d * %d\nfullscreen: %d", *w, *h, *fullscreen);                           // write them to the file
      
        fclose (config);                                                                            // close the file
      
      }
    
        else                                                                                        // write into the error log if the file couldn't be opened or created
          logerror (ERROR_FILE);

}



int menu ()
{

  int cursor = 0, first = 1; 
  // "cursor" = current position of the cursor
  // "first" = the user hasn't pressed a button so far
  Uint16 black;                                                                                     // black color
  SDL_Surface *title, *play, *highscores, *settings, *end, *version;                                // surfaces to render text
  SDL_keysym key;                                                                                   // used to store event, if a key was pressed
  SDL_Color fontcolor = {255, 255, 255}, chosen = {0, 255, 0};                                      // white and green color
  SDL_Rect dest;                                                                                    // destination of the surfaces
  SDL_Event event;                                                                                  // variable to store an event
  TTF_Font *font, *bigfont, *smallfont;                                                             // fonts to render text


  while (SDL_PollEvent (&event) != 0);                                                              // get all earlier events from the event queue

  while (1)                                                                                         // run this loop until the user pressed enter and "return" stops it
  {
    
    if (!first)                                                                                     // if it is the first frame, no events are captured
    {
    
      SDL_WaitEvent (&event);                                                                       // wait for an event
    
      switch (event.type)                                                                           // ask for event type
      {
    
        case SDL_KEYDOWN:                                                                           // a key was pressed
                           
                           key = event.key.keysym;                                                  // copy the event details to "key"
      
                           if (key.sym == SDLK_UP)                                                  // user pressed up-arrow
                           {
                          
                             if (cursor > 0)                                                        // if cursor is not on the first position
                               cursor--;                                                            // go to the upper position
                          
                               else                                                                 // if cursor is on the first position
                                 cursor = 3;                                                        // cursor is set on the last position
                          
                           }
                           
                             else if (key.sym == SDLK_DOWN)                                         // user pressed down-arrow
                             {
                             
                               if (cursor < 3)                                                      // if cursor is not on the last position
                                 cursor++;                                                          // go to the lower position
                             
                                 else                                                               // if cursor is on the last position
                                   cursor = 0;                                                      // cursor is set on the first position
                             
                             }
                             
                               else if (key.sym == SDLK_KP_ENTER || key.sym == SDLK_RETURN)         // user pressed enter
                                 return cursor;                                                     // return the decision
    
                                 else if (key.sym == SDLK_ESCAPE)                                   // escape was pressed
                                   return 3;                                                        // quits the game, but asks again
    
                           break;
                           
        case SDL_QUIT: exit (0);                                                                    // close was klicked
                       break;                                                                       // quits the game
    
      }  
      
    } 
    
    first = 0;                                                                                      // it is not the first frame anymore

    black = SDL_MapRGB (screen -> format, 0, 0, 0);                                                 // create a black color
    SDL_FillRect (screen, NULL, black);                                                             // the screen turns completely black    
 
                                                     
    bigfont = TTF_OpenFont ("arial.ttf", 60);                                                       // open 60 pt arial font
    font = TTF_OpenFont ("arial.ttf", 45);                                                          // open 45 pt arial font
    smallfont = TTF_OpenFont ("arial.ttf", 20);                                                     // open 20 pt arial font
 
    if (font != NULL && bigfont != NULL)                                                            // check if the fonts were opened correctly
    {      
  
      title = TTF_RenderText_Solid (bigfont, "EatTheBlocks", fontcolor);                            // render the title
    
    
      if (cursor == 0)                                                                              // current decision is "play"
        play = TTF_RenderText_Solid (font, "Play", chosen);                                         // it is rendered in a green font

        else                                                 
          play = TTF_RenderText_Solid (font, "Play", fontcolor);                                    // otherwise it is rendered in a white font       


      if (cursor == 1)                                                                              // current decision is "highscores"
        highscores = TTF_RenderText_Solid (font, "Highscores", chosen);                             // it is rendered in a green font

        else
          highscores = TTF_RenderText_Solid (font, "Highscores", fontcolor);                        // otherwise it is rendered in a white font


      if (cursor == 2)                                                                              // current decision is "settings"
        settings = TTF_RenderText_Solid (font, "Settings", chosen);                                 // it is rendered in a green font
        
        else
          settings = TTF_RenderText_Solid (font, "Settings", fontcolor);                            // otherwise it is rendered in a white font
          
          
      if (cursor == 3)                                                                              // current decision is "end"
        end = TTF_RenderText_Solid (font, "End", chosen);                                           // it is rendered in a green font
        
        else
          end = TTF_RenderText_Solid (font, "End", fontcolor);                                      // otherwise it is rendered in a white font
          

      version = TTF_RenderText_Solid (smallfont, CHARVERSION, fontcolor);                           // render version of the program in white font
      

      if (title && play && highscores && settings && end && version)                                // all surfaces were rendered successfully
      {
        
        dest.x = (screen -> w / 2) - (title -> w / 2);                                              // position of the title
        dest.y = 0;
        dest.w = title -> w;                                                                        // size of the title
        dest.h = title -> h;
      
        SDL_BlitSurface (title, NULL, screen, &dest);                                               // blit the surface onto the screen
      
      
        dest.y = (screen -> h / 3);                                                                 // poition of the surface
      
        dest.x = (screen -> w / 2) - (play -> w / 2); 
        dest.w = play -> w;                                                                         // size of the surface
        dest.h = play -> h;
      
        SDL_BlitSurface (play, NULL, screen, &dest);                                                // blit the surface onto the screen


        dest.y += 70;                                                                               // next surface is below the other one
       
        dest.x = (screen -> w / 2) - (highscores -> w / 2);                                         // position of the surface
        dest.w = highscores -> w;                                                                   // size of the surface
        dest.h = highscores -> h;
      
        SDL_BlitSurface (highscores, NULL, screen, &dest);                                          // blit surface onto the screen
               
               
        dest.y += 70;                                                                               // next surface is below the other one
      
        dest.x = (screen -> w / 2) - (settings -> w / 2);                                           // position of the surface
        dest.w = settings -> w;                                                                     // size of the surface
        dest.h = settings -> h;
      
        SDL_BlitSurface (settings, NULL, screen, &dest);                                            // blit surface onto the screen


        dest.y += 70;                                                                               // next surface is below the other one
      
        dest.x = (screen -> w / 2) - (end -> w / 2);                                                // position of the surface
        dest.w =  end  -> w;                                                                        // size of the surface
        dest.h =  end -> h;
      
        SDL_BlitSurface (end, NULL, screen, &dest);                                                 // blit surface onto the screen
      
        
        dest.x = (screen -> w) - (version -> w) - 5;                                                // position of the surface
        dest.y = (screen -> h) - (version -> h);
        dest.w = version -> w;                                                                      // size of the surface
        dest.h = version -> h;
        
        SDL_BlitSurface (version, NULL, screen, &dest);                                             // blit surface onto the screen
      
      
        SDL_Flip (screen);                                                                          // flip the screen (doublebuffer)
      
        SDL_FreeSurface (title);                                                                    // free the rendered surfaces
        SDL_FreeSurface (play);
        SDL_FreeSurface (highscores);
        SDL_FreeSurface (settings);
        SDL_FreeSurface (end);
        SDL_FreeSurface (version);
          
      }

        else                                                                                        // write into the error log if the surfaces weren't rendered correctly
          logerror (ERROR_TTF);

        
      TTF_CloseFont (font);                                                                         // close the fonts
      TTF_CloseFont (bigfont);
      TTF_CloseFont (smallfont);
   
    }
  
      else                                                                                          // write into the error log if the fonts weren't opened correctly
        logerror (ERROR_TTF);     

  }

  return 0;

}



void game ()                                                                                        // starts the game
{

  char charagain[] = "Play again?";                                                                 // text to ask the player if he wants to play again

  do
  {
  
    starting_snake_fruit ();                                                                        // creates the snake and fruit for the beginning

    main_loop ();                                                                                   // main game loop

    deletesnake ();                                                                                 // frees the allocated memory

  } while (yesorno (charagain));                                                                    // asks the player to play again

}



void settings (int w, int h, int fullscreen)                                                        // change settings
{
 
  SDL_Surface *res, *resvalue, *fullscr, *fullscrvalue, *ok, *cancel;                               // surfaces to render
  SDL_Rect dest;                                                                                    // destination of the surfaces
  SDL_keysym key;                                                                                   // used to store event, if a key was pressed
  SDL_Color fontcolor = {255, 255, 255}, chosen = {0, 255, 0};                                      // white and green color
  SDL_Event event;                                                                                  // variable to store an event
  TTF_Font *font;                                                                                   // font to render text
  resmodes modes[] = {{800, 600}, {1024, 768}, {1360, 768}, {1440, 900}, {1920, 1200}};             // supported resolutions
  Uint16 black;                                                                                     // black color
  int cursor = 0, modenr = 0, nummodes = 5, run = 1, first = 1;                                     
  // "cursor" = current position of the cursor
  // "nodenr" = resolution number (1 for 800*600, 2 for 1024*768, ...)
  // "nummodes" = number of supported resolutions
  // "run" = variable to control the loop
  // "first" = says if it is the first frame or not
  char charres[10], charfullscr[4];                                                                 // strings containing the resolution and if fullscreen or not
  char title[] = "SETTINGS";                                                                        // title of the screen
  
  
  switch (w)                                                                                        // determine current resolution
  {
  
    case 800: modenr = 0;                                                                           // resolution is 800*600, number 0
              break;
              
     case 1024: modenr = 1;                                                                         // resolution is 1024*768, number 1
                break;
                
     case 1360: modenr = 2;                                                                         // resolution is 1360*768, number 2
                break;
      
      case 1440: modenr = 3;                                                                        // resolution is 1440*900, number 3
                 break;
                 
      case 1920: modenr = 4;                                                                        // resolution is 1920*1200, number 4
                 break;
                 
       default: modenr = 0;                                                                         // other resolution, number 0
  
  }
  
  
  while (SDL_PollEvent (&event) != 0);                                                              // get all earlier events from the event queue

  while (run)                                                                                       // run this loop until the user pressed escape or chose enter or cancel
  {
    
    if (!first)                                                                                     // if it is the first frame, no events are captured
    {
    
      SDL_WaitEvent (&event);                                                                       // wait for an event
    
      switch (event.type)                                                                           // ask for event type
      {
    
        case SDL_KEYDOWN:                                                                           // a key was pressed
                           
                           key = event.key.keysym;                                                  // copy the event details to "key"
      
                           if (key.sym == SDLK_UP && cursor > 0)                                    // user pressed up-arrow
                             cursor--;                                                              // cursor moves a position up
                           
                             else if (key.sym == SDLK_DOWN && cursor < 3)                           // user pressed down-arrow
                               cursor++;                                                            // curosr moves a position down
                             
                               else if (key.sym == SDLK_LEFT)                                       // left arrow-key was pressed
                               {
                                 
                                 if (cursor == 0)                                                   // cursor is on "resolution"
                                 {
                                 
                                   if (modenr > 0)                                                  // check if it is not the first resolution mode
                                   {
                                   
                                     modenr--;                                                      // choose other resolution mode
                                   
                                     w = modes[modenr].w;                                           // assign new resolution
                                     h = modes[modenr].h;
                                   
                                   }
                               
                                 }
                               
                                 else if (cursor == 1)                                              // cursor is on "fullscreen"
                                   fullscreen = 0;                                                  // fullscreen indicator is 0
                               
                               } 
                                 
                                 else if (key.sym == SDLK_RIGHT)                                    // right arrow-key was pressed
                                 {
                                 
                                   if (cursor == 0)                                                 // cursor is on "resolution"
                                   {
                                   
                                     if (modenr < nummodes - 1)                                     // if it is not the last mode
                                     {
                                     
                                       modenr++;                                                    // next resolution
                                     
                                       w = modes[modenr].w;                                         // assign new resolution
                                       h = modes[modenr].h;
                                     
                                     }
                                   
                                   } 
                                       
                                   else if (cursor == 1)                                            // cursor is on "fullscreen"
                                     fullscreen = 1;                                                // fullscreen indicator is 1
                                 
                                 }
                            
                                   else if (key.sym == SDLK_KP_ENTER || key.sym == SDLK_RETURN)     // enter was pressed
                                   {
                                    
                                     if (cursor == 2)                                               // cursor on ok
                                     {
                                       
                                       load_config (&w, &h, &fullscreen, 0);                        // writes new settings to the configuration file
                                       
                                       if (fullscreen)                                              // fullscreen is activated
                                         screen = SDL_SetVideoMode (w, h, 16,                       // change resolution and to fullscreen
                                                                    SDL_DOUBLEBUF | SDL_FULLSCREEN);   
                                         
                                         else
                                           screen = SDL_SetVideoMode (w, h, 16, SDL_DOUBLEBUF);     // change resolution and to non-fullscreen
                                       
                                       run = 0;                                                     // stops the loop
                                       
                                     }
                                                                    
                                       else if (cursor == 3)                                        // cursor is on cancel
                                         run = 0;                                                   // stops the loop
                                                                               
                                   }                                    
                                   
                                     else if (key.sym == SDLK_ESCAPE)                               // escape was pressed
                                       run = 0;                                                     // stops the loop
    
                           break;
                           
        case SDL_QUIT: exit (0);                                                                    // close was klicked
                       break;                                                                       // quits the game
    
      }  
      
    } 

    
    first = 0;                                                                                      // it is not the first frame anymore

    black = SDL_MapRGB (screen -> format, 0, 0, 0);                                                 // create a black color
    SDL_FillRect (screen, NULL, black);                                                             // the screen turns completely black    
 
                                                     
    font = TTF_OpenFont ("arial.ttf", 30);                                                          // open 30 pt arial font
 
    if (font != NULL)                                                                               // check if font loaded correctly
    {      
  
      draw_title (title);                                                                           // draws title onto the screen
      
    
      sprintf (charres, "%d*%d", w, h);                                                             // copy the resolution into the string
      
      res = TTF_RenderText_Solid (font, "Resolution", fontcolor);                                   // render the word "Resolution"
    
      if (cursor == 0)                                                                              // current cursor position is "resolution"
        resvalue = TTF_RenderText_Solid (font, charres, chosen);                                    // it is rendered in a green font

        else                                                 
          resvalue = TTF_RenderText_Solid (font, charres, fontcolor);                               // otherwise it is rendered in a white font       


      if (fullscreen == 0)                                                                          // fullscreen indicator is 0
        strcpy (charfullscr, "OFF");                                                                // write "OFF" into the string
        
        else
          strcpy (charfullscr, "ON");                                                               // write "ON" into the string

      fullscr = TTF_RenderText_Solid (font, "Fullscreen", fontcolor);                               // render text to surface

      if (cursor == 1)                                                                              // current position is "fullscreen"
        fullscrvalue = TTF_RenderText_Solid (font, charfullscr, chosen);                            // it is rendered in a green font

        else
          fullscrvalue = TTF_RenderText_Solid (font, charfullscr, fontcolor);                        // otherwise it is rendered in a white font


      if (cursor == 2)                                                                              // current position is "ok"
        ok = TTF_RenderText_Solid (font, "OK", chosen);                                             // it is rendered in a green font
        
        else
          ok = TTF_RenderText_Solid (font, "OK", fontcolor);                                        // otherwise it is rendered in a white font
          
          
      if (cursor == 3)                                                                              // current decision is "cancel"
        cancel = TTF_RenderText_Solid (font, "Cancel", chosen);                                     // it is rendered in a green font
        
        else
          cancel = TTF_RenderText_Solid (font, "Cancel", fontcolor);                                // otherwise it is rendered in a white font
         

      if (res && fullscr && ok && cancel)                                                  // all surfaces were rendered successfully
      {      
      
        dest.y = (screen -> h / 3);                                                                 // poition of the surface
      
        dest.x = (screen -> w / 5); 
        dest.w = res -> w;                                                                          // size of the surface
        dest.h = res -> h;
      
        SDL_BlitSurface (res, NULL, screen, &dest);                                                 // blit the surface onto the screen

        dest.x += screen -> w / 2;                                                                  // position of the surface
        dest.w = resvalue -> w;                                                                     // size of the surface
        dest.h = resvalue -> h;
        
        SDL_BlitSurface (resvalue, NULL, screen, &dest);                                            // blit the surface onto the screen


        dest.y += 70;                                                                               // next surface is below the other one
       
        dest.x = (screen -> w / 5);                                                                 // position of the surface
        dest.w = fullscr -> w;                                                                      // size of the surface
        dest.h = fullscr -> h;
      
        SDL_BlitSurface (fullscr, NULL, screen, &dest);                                             // blit surface onto the screen
             
        dest.x += screen -> w / 2;                                                                  // position of the surface
        dest.w = fullscrvalue -> w;                                                                 // size of the surface
        dest.h = fullscrvalue -> h;
        
        SDL_BlitSurface (fullscrvalue, NULL, screen, &dest);                                        // blit surface onto the screen
               
               
        dest.x = (screen -> w / 2) - (ok -> w / 2);                                                 // position of the surface
        dest.y = (screen -> h) - (ok -> h * 4);
        dest.w = ok -> w;                                                                           // size of the surface
        dest.h = ok -> h;
        
        SDL_BlitSurface (ok, NULL, screen, &dest);                                                  // blit surface onto the screen
        
        
        dest.x = (screen -> w / 2) - (cancel -> w / 2);                                             // position of the surface
        dest.y = (screen -> h) - (cancel -> h * 2 + (cancel -> h / 2));
        dest.w = cancel -> w;                                                                       // size of the surface
        dest.h = cancel -> h;
        
        SDL_BlitSurface (cancel, NULL, screen, &dest);                                              // blit surface onto the screen
   
      
        SDL_Flip (screen);                                                                          // flip the screen (doublebuffer)
                                                                   
        SDL_FreeSurface (res);                                                                      // free the rendered surfaces
        SDL_FreeSurface (resvalue);
        SDL_FreeSurface (fullscr);
        SDL_FreeSurface (fullscrvalue);
        SDL_FreeSurface (ok);
        SDL_FreeSurface (cancel);
          
      }

        else                                                                                        // write into the error log if the surfaces weren't rendered correctly
          logerror (ERROR_TTF);      
        
      TTF_CloseFont (font);                                                                         // close the fonts
   
    }
  
      else                                                                                          // write into the error log if the fonts weren't opened correctly
        logerror (ERROR_TTF);  

  }

}



void main_loop ()                                                                                   // main game loop
{

  int lasttime, run = 1;                                                                            // time starting the frame; "run" used to stop the program
  unsigned int score = 0;                                                                           // reached score
  

  while (run)                                                                                       // run until something happens that "run" is changed
  {

    lasttime = SDL_GetTicks () + 100;                                                               // get the time since SDL was started
    frames++;                                                                                       // increase number of frames
    
    check_events (&run);                                                                            // check for SDL events

    rendering ();                                                                                   // render background, fruit and snake

    check_border_score (&run, &score);                                                              // check if the snake hit a border are ate a fruit

    save_locations ();                                                                              // save the coordinations of the snake
    
    calculate_speed (&lasttime);                                                                    // calculates speed

    move_snake ();                                                                                  // the snake gets a new position

  }

}



void parameters (int *w, int *h, int *fullscreen, int argc, char *argv[])                           // check command line parameters
{

  int i;                                                                                            // counter variable

  for (i = 1; i < argc; i++)                                                                        // check the arguments, the first (path + filename) is ignored
  {

    if (parameter_width (argv[i], argv[i + 1], w));                                                 // check for "-w" flag to define the window width

    else if (parameter_height (argv[i], argv[i + 1], h));                                           // check for "-h" flag to define the window height

    else if (parameter_fullscreen (argv[i], fullscreen));                                           // check for "--full" flag to start game in fullscreen mode

    else parameter_help (argv[i]);                                                                  // check for "-h" flag to display informations in the terminal

  }

}



int parameter_width (char *flag, char *par, int *w)                                                 // check for "-w" flag to define the window width
{

  if (strcmp (flag, "-w") == 0)                                                                     // compares parameter with "-w" to define the window width
  {

    if (atoi (par) >= 800)                                                                          // check if window is wide enough
    {

      *w = atoi (par);                                                                              // assign the width

      return 1;                                                                                     // return 1 to indicate that this flag was available

    }

      else                                                                                          // write into the error log if the window is too small
      {

        printf ("Window too small.\n");

        exit (0);                                                                                   // exit the program

      }

  }

  return 0;                                                                                         // return 0 to indicate that this flag was not available

}



int parameter_height (char *flag, char *par, int *h)                                                // check for "-h" flag to define the window height
{

  if (strcmp (flag, "-h") == 0)                                                                     // compares parameter with "-h" to define the window height
  {

    if (atoi (par) >= 600)                                                                          // check if window is high enough
    {

      *h = atoi (par);                                                                              // assign the height

      return 1;                                                                                     // return 1 to indicate that this flag was available

    }

      else                                                                                          // write into the error log if the window is too small
      {

        printf ("Window too small.\n");

        exit (0);                                                                                   // exit the program

      }

  }

  return 0;                                                                                         // return 0 to indicate that this flag was not available

}



int parameter_fullscreen (char *flag, int *fullscreen)                                              // check for "--full" flag to start game in fullscreen mode
{

  if (strcmp (flag, "--full") == 0)                                                                 // compare parameter with "--full" flag to start game in fullscreen mode
  {

    *fullscreen = 1;                                                                                // use fullscreen

    return 1;                                                                                       // return 1 to indicate that this flag was available

  }

  return 0;                                                                                         // return 0 to indicate that this flag was not available

}



void parameter_help (char *flag)                                                                    // check for "-help" flag to display informations in the terminal
{

  if (strcmp (flag, "--help") == 0)                                                                 // compare parameter with "--help" to display informations in the terminal
  {

    display_help ();                                                                                // print help

    exit (0);                                                                                       // quit the program

  }

}



void display_help ()                                                                                // displays informations in the terminal
{

  printf ("\nEatTheBlocks V%.1f%c\n", VERSION, SUBVERSION);                                         // prints version of the game

  printf ("\nOptions:\n\t-w\t=\tspecify windows width\n");                                          // prints available flags and their usage
  printf ("\t-h\t=\tspecify window hight\n\t--full\t");
  printf ("=\tstart game in fullscreen mode\n\t--help\t=\tshow this help\n");

}



int start_SDL (int w, int h, int fullscreen)                                                        // initializes SDL and TTF and sets video mode
{

  if (SDL_Init (SDL_INIT_VIDEO) != 0)                                                               // initialize SDL videosubsystem and check if it was initialized correct
  {

   logerror (ERROR_SDL);

    return 1;                                                                                       // stops the program

  }

  atexit (SDL_Quit);                                                                                // quit SDL at the end of the program


  if (TTF_Init () != 0)                                                                             // initialize TTF and check if it was initialized correct
  {

    logerror (ERROR_TTF);

    return 1;                                                                                       // stops the program

  }
  
  atexit (TTF_Quit);                                                                                // quit TTF at the end of the program


  SDL_WM_SetCaption ("EatTheBlocks", "EatTheBlocks");                                               // set window title to "EatTheBlocks"
  SDL_ShowCursor (SDL_DISABLE);                                                                     // hide the cursor on the window


  if (fullscreen)                                                                                   // check if fullscreen shall be enabled
    screen = SDL_SetVideoMode (w, h, 16, SDL_DOUBLEBUF | SDL_FULLSCREEN);                           // set SDL to fullscreen

    else
      screen = SDL_SetVideoMode (w, h, 16, SDL_DOUBLEBUF);                                          // use a window (standard 800 * 600)


  if (screen == NULL)                                                                               // check if video mode was set correctly
  {

    logerror (ERROR_SDL);

    return 1;                                                                                       // stops the program

  }

  return 0;                                                                                         // everything went right

}



void draw_snake ()                                                                                  // draws the snake on the surface
{

  int i;                                                                                            // counter variable
  SDL_Rect dest;                                                                                    // stores destination coordinates of the blocks
  Uint16 white;                                                                                     // stores colorcode for white
  block_info *help;
  
  help = snake;                                                                                     // "help" is at the head of the snake 

  dest.w = 10;                                                                                      // width is 10 pixels, equal for all blocks
  dest.h = 10;                                                                                      // height is 10 pixels, equal for all blocks

  white = SDL_MapRGB (screen -> format, 255, 255, 255);                                             // get a white color

  for (i = 0; i < length; i++)                                                                      // loop drawing blocks
  {

    dest.x = help -> x;                                                                             // the destination is the x and y coordinate of the block
    dest.y = help -> y;

    SDL_FillRect (screen, &dest, white);                                                            // priont a white block to the screen
  
    help = help -> next;                                                                            // draw next block

  }

}



void draw_fruit ()                                                                                  // draws the fruit
{

  SDL_Rect dest;                                                                                    // stores destination coordinates of the fruit
  Uint16 green;                                                                                     // stores colorcode for green

  dest.x = fruit.x;                                                                                 // the destination is the x and y coordinate of the fruit
  dest.y = fruit.y;
  dest.w = 10;                                                                                      // the fruit is a 10 * 10 pixel block
  dest.h = 10;

  green = SDL_MapRGB (screen -> format, 0, 255, 0);                                                 // get a green color
  SDL_FillRect (screen, &dest, green);                                                              // print a green fruit to the screen

}



void new_fruit ()                                                                                   // get coordinates of a fruit
{

  do
  {

    fruit.x = (float) ((rand () % (screen -> w - 20)) + 5);                                         // fruit is at least 5 pixels away from the border
                                                                                                    // 20: 10 for block width and 10 for distance from the down/right border
    fruit.y = (float) ((rand () % (screen -> h - 20)) + 5);                                         // 5 for distance from the upper/left border and to get 5 on the upper/right

  } while (onsnake (fruit.x, fruit.y));                                                             // get new values until the fruit is not on the snake

}



int onsnake (float x, float y)                                                                      // check if fruit is on the snake
{

  block_info *help;                                                                                 // help pointer
  
  help = snake;                                                                                     // help pointer is on the first block

  while (help != NULL)                                                                              // go until the end of the snake is reached
  {

    if ((x < (help -> x + 10) && x > (help -> x - 10)))                                             // check x-coordinates of the fruit
    {

      if ((y < (help -> y + 10) && y > (help -> y - 10)))                                           // check y-coordinates of the fruit
      {
                                                 
        return 1;                                                                                   // fruit is on the snake

      }

    }
    
    help = help -> next;                                                                            // check next block

  }  
  
  return 0;                                                                                         // fruit is not on the snake

}



void check_input (SDL_Event event, int *run)                                                        // checks the input of an event
{

  SDL_keysym key = event.key.keysym;                                                                // get the keyname

  if (key.sym == SDLK_UP && snake -> direction != DOWN)                                             // up pressed and the snake is not moving down
    snake -> direction = UP;                                                                        // the snake will move up next frame

  else if (key.sym == SDLK_RIGHT && snake -> direction != LEFT)                                     // right pressed and the snake is not moving left
    snake -> direction = RIGHT;                                                                     // the snake will move right next frame

  else if (key.sym == SDLK_DOWN && snake -> direction != UP)                                        // down pressed and the snake is not moving up
    snake -> direction = DOWN;                                                                      // the snake will move down next frame

  else if (key.sym == SDLK_LEFT && snake -> direction != RIGHT)                                     // left pressed and the snake is not moving right
    snake -> direction = LEFT;                                                                      // the snake will move left next frame

  else if (key.sym == SDLK_p)                                                                       // if 'p' was pressed
    pause (run);                                                                                    // the game is paused and run is passed

  else if (key.sym == SDLK_ESCAPE)                                                                  // escape was pressed
    *run = 0;                                                                                       // stops the game

}



void check_border_score (int *run, unsigned int *score)                                             // checks if the snake hit the border, ate a fruit or bit itself
{

  int bitself;                                                                                      // variable to check if the snake bit itself


  bitself = bit_self ();                                                                            // check if the snake bit itself

  check_hit_border (bitself, *score, run);                                                          // check if snake hit the border

  check_ate_fruit (score);                                                                          // check if the snake ate a fruit

}



void move_snake ()                                                                                  // moves the blocks to their new position
{

  int i;                                                                                            // counter variable
  block_info *help, *oldhelp;

  help = snake;                                                                                     // "help" is at the head of the snake
  oldhelp = oldpos;                                                                                 // "oldhelp" is at the beginning of the list


  for (i = 0; i < length; i++)                                                                      // loop to move all blocks
  {

    if (i == 0)                                                                                     // for the first block ("head" of the snake)
    {

      if (snake -> direction == UP)                                                                 // if the direction of the head is up
        snake -> y -= 25.0 * speed;                                                                 // the y-coodinate of the block is decreased

      else if (snake -> direction == RIGHT)                                                         // if the direction of the head is right
        snake -> x += 25.0 * speed;                                                                 // the x-coordinate of the block is increased

      else if (snake -> direction == DOWN)                                                          // if the direction of the head is up
        snake -> y += 25.0 * speed;                                                                 // the y-coordinate of the block is decreased

      else if (snake -> direction == LEFT)                                                          // if the direction of the head is left
        snake -> x -= 25.0 * speed;                                                                 // the x-coordinate of the block is decreased

    }

      else                                                                                          // for all other blocks
      {
        help -> direction = oldhelp -> prev -> direction;                                           // each block gets the position and direction of the
        help -> x = oldhelp -> prev -> x;                                                           // forerunner. this informations were saved in "save_locations"
        help -> y = oldhelp -> prev -> y;
      }
      
    help = help -> next;                                                                            // next block
    oldhelp = oldhelp -> next;                                                                      // next positions

  }

}



void pause (int *run)                                                                               // if 'p' is pressed the game is paused
{

  int pausegame = 1;                                                                                // variable to let the loop run
  SDL_Event event;                                                                                  // variable to store an event
  SDL_keysym key;                                                                                   // variable to store a key


  pause_screen ();                                                                                  // draws pause onto the screen


  while (pausegame)                                                                                 // run loop until an event stops it
  {

    while (SDL_PollEvent (&event))                                                                  // get the events
    {

      switch (event.type)                                                                           // ask for the event type
      {

        case SDL_KEYDOWN:                                                                           // a key was pressed

                          key = event.key.keysym;                                                   // gets the keyname

                          if (key.sym == SDLK_p)                                                    // 'p' was pressed
                            pausegame = 0;                                                          // stops the break

                            else if (key.sym == SDLK_ESCAPE)                                        // if escape was pressed
                            {
                              *run = 0;                                                             // the program will stop
                              pausegame = 0;                                                        // the pause-loop is stopped
                            }

                          break;

        case SDL_QUIT: *run = 0;                                                                    // if the user klicked "close" the program is stopped
                       pausegame = 0;                                                               // the pause-loop is stopped
                       break;

      }

    }

  }

}



void create_blocks (int nr)                                                                         // appends new blocks to the snake; "nr" = number of new blocks
{                                                                                             

  int i;                                                                                            // counter variable
  block_info *help, *oldhelp, *newblock;                                                            // help pointers to allocate memory and go through the list
  
  help = snake;                                                                                     // "help" is at the beginning of the snake
  oldhelp = oldpos;                                                                                 // "oldhelp" is at the beginning of the list with the old positions
  
  
  while (help -> next != NULL && oldhelp -> next != NULL)                                           // set both pointers to the last valid block
  {
    help = help -> next;                                                                            // next block
    oldhelp = oldhelp -> next;                                                                      // next block
  } 
 

  for (i = 0; i < nr && length < MAXLENGTH; i++, length++)                                          // add "nr" elements; number of blocks must be smallter than MAXLENGTH
  {
  
    newblock = (block_info *) malloc (sizeof (block_info));                                         // allocate memory for a new block
    newblock -> next = NULL;                                                                        // has no follower
    newblock -> prev = oldhelp;                                                                     // the previous block is "oldhelp"
  
    oldhelp -> next = newblock;                                                                     // append new block at the end of the list
    oldhelp = newblock;                                                                             // set pointer to the last block
        
       
    newblock = (block_info *) malloc (sizeof (block_info));                                         // allocate memory for a new block
    newblock -> next = NULL;                                                                        // has no follower
    newblock -> prev = help;                                                                        // the previous block is "help"
  
    help -> next = newblock;                                                                        // append new block at the end of the list
    help = newblock;                                                                                // set pointer to the last block


    help -> direction = help -> prev -> direction;                                                  // new block gets the direction of its forerunner


    if (help -> direction == UP)                                                                    // if the direction is up
    {

      help -> x = help -> prev -> x;                                                                // the new block has the same x-coordinates as its forerunner

      help -> y = help -> prev -> y + 1.0;                                                          // the new block is placed below its forerunner

    }

      else if (help -> direction == DOWN)                                                           // if the direction is down
      {

        help -> x = help -> prev -> x;                                                              // the new block has the same x-coordinate as its forerunner

        help -> y = help -> prev -> y - 1.0;                                                        // the new block is placed above its forerunner

      }

        else if (help -> direction == RIGHT)                                                        // if the direction is right
        {

          help -> x = help -> prev -> x - 1.0;                                                      // the new block is placed left of its forerunner

          help -> y = help -> prev -> y;                                                            // the new block has the same y-coordinate as its forerunner

        }

          else if (help -> direction == LEFT)                                                       // if the direction is left
          {

            help -> x = help -> prev -> x + 1.0;                                                    // the new block is placed right of its forerunner

            help -> y = help -> prev -> y;                                                          // the new block has the same y-coordinate as its forerunner

          }

  }

}



void draw_end (unsigned int score)                                                                  // draw score and "game over" at the end of the game
{

  SDL_Surface *text, *gameover;                                                                     // surfaces for the score and "game over"
  SDL_Rect dest;                                                                                    // destination of the surfaces
  SDL_Color color = {0, 255, 0}, color2 = {255, 255, 255};                                          // green and white fontcolor
  TTF_Font *font;                                                                                   // fonts for the text
  Uint16 rectcolor;                                                                                 // color of "game over" background
  char charscore[19], buffer[10];                                                                   // strings for the score


  strcpy (charscore, "Score: \0");                                                                 // "Score: " is copied to "charscore"
  sprintf (buffer, "%d", score);                                                                    // the score is written to "buffer"
  strcat (charscore, buffer);                                                                       // the score is appended to the string


  rectcolor = SDL_MapRGB (screen -> format, 0, 0, 0);                                               // get a black color
  SDL_FillRect (screen, NULL, rectcolor);                                                           // the screen turns completely black


  draw_blockwithframe ((screen -> w / 2) - 105, (screen -> h / 2) - 20, 210, 40);                   // draws a white frame; the details to the position are passed

  font = TTF_OpenFont ("arial.ttf", 35);                                                            // get a 35 pt arial font


  if (font != NULL)                                                                                 // check if fonts were opened correctly
  {

    text = TTF_RenderText_Solid (font, charscore, color);                                           // render surface for the score in green color in the left top corner
    gameover = TTF_RenderText_Solid (font, "GAME OVER", color2);                                    // render surface for "game over" in red color in the middle of the screen

    if (text != NULL && gameover != NULL)                                                           // check if the surfaces were rendered correctly
    {

      dest.x = 5;                                                                                   // position of the surface
      dest.y = 0;
      dest.w = text -> w;                                                                           // size of the surface
      dest.h = text -> h;

      SDL_BlitSurface (text, NULL, screen, &dest);                                                  // blits the score onto the screen


      dest.x = (screen -> w / 2) - (gameover -> w / 2);                                             // coordinates of the black rectangle
      dest.y = (screen -> h / 2) - (gameover -> h / 2);
      dest.w = gameover -> w;                                                                       // width of the rectangle
      dest.h = gameover -> h;                                                                       // height of the rectangle

      SDL_BlitSurface (gameover, NULL, screen, &dest);                                              // blits "game over" onto the screen

      SDL_Flip (screen);                                                                            // flip surfaces (doublebuffer)

      SDL_Delay (2000);                                                                             // wait 3 seconds

      SDL_FreeSurface (text);                                                                       // free both surfaces
      SDL_FreeSurface (gameover);

    }

      else                                                                                          // write into the error log if the surfaces weren't rendered correctly
        logerror (ERROR_TTF);

    TTF_CloseFont (font);                                                                           // close both fonts

  }

    else                                                                                            // write into the error log if the fonts couldn't be opened correctly
      logerror (ERROR_TTF);

}



void save_locations ()                                                                              // saves the location and direction of the blocks
{

  int i;                                                                                            // counter variable
  block_info *help, *oldhelp;                                                                       // help pointers to go through lists
  
  help = snake;                                                                                     // "help" is at the beginning of the snake
  oldhelp = oldpos;                                                                                 // "oldhelp" is at the beginning of the list with the old positions  


  for (i = 0; i < length; i++)                                                                      // copy the position of all blocks
  {
  
    oldhelp -> direction = help -> direction;                                                       // copy direction to the other list
    oldhelp -> x = help -> x;                                                                       // copy the x-coordinate to the other list
    oldhelp -> y = help -> y;                                                                       // copy the y-coordinate to the other list

    help = help -> next;                                                                            // go to next block
    oldhelp = oldhelp -> next;                                                                      // go to next block

  }

}



int bit_self ()                                                                                     // checks if the snake bit itself
{

  int i;                                                                                            // counter variable
  block_info *help;                                                                                 // help pointer to go through list

  help = snake;                                                                                     // "help" is at the beginning of the snake

  for (i = 0; i < 10; i++)                                                                          // go to 10 th block
    help = help -> next;                                                                            // next block


  for (i = 10; i < length; i++)                                                                     // check from the 10 th block on
  {

    if ((snake -> x < (help -> x + 10) && snake -> x > (help -> x - 10)))                           // check the x-coordinate of the head
    {

      if ((snake -> y < (help -> y + 10) && snake -> y > (help -> y - 10)))                         // check the y-coordinate of the head
      {

        if (snake -> direction != help -> direction)                                                // check if the blocks have not the same direction
          return 1;                                                                                 // return 1 --> bit itself

      }

    }
    
    help = help -> next;                                                                            // check next block

  }

  return 0;                                                                                         // didn't bite itself

}



void pause_screen ()                                                                                // draws a pause screen
{

  SDL_Surface *pause;                                                                               // surface for the text
  SDL_Color fontcolor = {255, 255, 255};                                                            // fontcolor is white
  SDL_Rect dest;
  TTF_Font *font;                                                                                   // font for the text


  draw_blockwithframe ((screen -> w / 2) - 105, (screen -> h / 2) - 20, 210, 40);                   // draws a white frame; details to the position are passsed

  font = TTF_OpenFont ("arial.ttf", 35);                                                            // open 35 pt arial font


  if (font != NULL)                                                                                 // check if font opened correctly
  {

    pause = TTF_RenderText_Solid (font, "Pause", fontcolor);                                        // render white "Pause" text

    dest.x = (screen -> w / 2) - (pause -> w / 2);                                                  // coordinates of the black rectangle
    dest.y = (screen -> h / 2) - (pause -> h / 2);
    dest.w = pause -> w;                                                                            // width of the rectangle
    dest.h = pause -> h;                                                                            // height of the rectangle

    if (pause != NULL)                                                                              // check if rendered correctly
    {

      SDL_BlitSurface (pause, NULL, screen, &dest);                                                 // blit surface onto the screen

      SDL_Flip (screen);                                                                            // flip surface (doublebuffer)

      SDL_FreeSurface (pause);                                                                      // free the surface

    }

      else                                                                                          // write into the error log if the surfaces weren't rendered correctly
        logerror (ERROR_TTF);

    TTF_CloseFont (font);                                                                           // close the font

  }

    else                                                                                            // write into the error log if the fonts weren't opened correctly
      logerror (ERROR_TTF);
 
}



void rendering ()                                                                                   // render background, fruit and snake
{

  Uint16 black;                                                                                     // variable to store colorcode for black

  black = SDL_MapRGB (screen -> format, 0, 0, 0);                                                   // create a black color variable


  SDL_FillRect (screen, NULL, black);                                                               // fill the whole screen black

  draw_snake ();                                                                                    // draw the snake

  draw_fruit ();                                                                                    // draw the fruit

  SDL_Flip (screen);                                                                                // flip the screen (doublebuffer)

}



void calculate_speed (int *lasttime)                                                                // calculates speed
{

  if((SDL_GetTicks () - *lasttime) >= 100)                                                          // all 100 ms
  {

    fps = frames * 1000 / 100;                                                                      // calculate frames per second
    frames = 0;                                                                                     // set "frames" to 0 again

    speed = 1 / fps * level;                                                                        // calculate "speed"

    *lasttime = SDL_GetTicks ();                                                                    // get mew time

  }

}



void check_events (int *run)                                                                        // check for SDL events
{

  SDL_Event event;                                                                                  // event variable to store events

  while (SDL_PollEvent (&event))                                                                    // get events
  {

    switch (event.type)                                                                             // ask for the type
    {

      case SDL_KEYDOWN: check_input (event, run);                                                   // if a key was pressed check which one
                        break;

      case SDL_QUIT: *run = 0;                                                                      // if the user klicked "close" the program is stopped
                     break;

    }

  }

}



void starting_snake_fruit ()                                                                        // creates the snake and fruit for the beginning
{

  Uint16 black;                                                                                     // variable to store colorcode for black

  black = SDL_MapRGB (screen -> format, 0, 0, 0);                                                   // create a black color variable

  SDL_FillRect (screen, NULL, black);                                                               // fill the whole screen black

  length = 0;                                                                                       // the snake has 0 blocks
  
  level = 1.0;                                                                                      // level 1.0
  
  snake = (block_info *) malloc (sizeof (block_info));                                              // allocate memory for new block

  snake -> x = (float) (screen -> w / 2) - 5.0;                                                     // set the x-position of the first block
  snake -> y = (float) (screen -> h / 2) - 5.0;                                                     // set the y-position of the first block
  snake -> direction = LEFT;                                                                        // move left at the beginning

  snake -> next = NULL;                                                                             // block has no follower
  snake -> prev = NULL;                                                                             // block has no forerunner

  length++;                                                                                         // increase the length


  oldpos = (block_info *) malloc (sizeof (block_info));                                             // allocate memory for new block
  
  oldpos -> next = NULL;                                                                            // block has no follower
  oldpos -> prev = NULL;                                                                            // block has no forerunner


  create_blocks (20);                                                                               // create 20 new blocks

  new_fruit ();                                                                                     // get position of a new fruit

}



void check_hit_border (int bitself, unsigned int score, int *run)                                   // check if snake hit the border
{

  if ((snake -> x < 0) || ((snake -> x + 10) > screen -> w) || bitself)                             // if the first block hit the right or left border or the snake bit itself
  {

    *run = 0;                                                                                       // the loop in main and so the program is stopped

    draw_end (score);                                                                               // the score and "game over" are drawn onto the screen

    highscore (score);                                                                              // the highscores are drawn onto the screen

  }

    else if ((snake -> y < 0) || ((snake -> y + 10) > screen -> h) || bitself)                      // hit the border on top or bottom or the snake bit itself
    {

      *run = 0;                                                                                     // the loop in main and so the program is stopped

      draw_end (score);                                                                             // the score and "game over" are drawn onto the screen

      highscore (score);                                                                            // the highscores are drawn onto the screen

    }

}



void check_ate_fruit (unsigned int *score)                                                          // check if the snake ate a fruit
{

  if ((snake -> x < (fruit.x + 10) && snake -> x > (fruit.x - 10)))                                 // check if the first block's x-coordinates are on the fruit
  {

    if ((snake -> y < (fruit.y + 10) && snake -> y > (fruit.y - 10)))                               // check if the first block's y-coordinates are on the fruit
    {

      (*score)++;                                                                                   // increase score

      create_blocks (5);                                                                            // creates 5 new blocks

      new_fruit ();                                                                                 // get new coordinates for the fruit

      level += 0.01;                                                                                // increase the level

    }

  }


}



void draw_blockwithframe (int x, int y, int w, int h)                                               // draws a black block with a white frame
{

  SDL_Rect dest;                                                                                    // destination of the surface
  Uint16 color;                                                                                     // color of the rects


  color = SDL_MapRGB (screen -> format, 255, 255, 255);                                             // get a white color

  dest.x = x - 2;                                                                                   // coordinates of the white rectangle
  dest.y = y - 2;
  dest.w = w + 4;                                                                                   // width of the rectangle
  dest.h = h + 4;                                                                                   // height of the rectangle

  SDL_FillRect (screen, &dest, color);                                                              // draw the white block to the screen


  color = SDL_MapRGB (screen -> format, 0, 0, 0);                                                   // get a black color

  dest.x = x;                                                                                       // coordinates of the black rectangle
  dest.y = y;
  dest.w = w;                                                                                       // width of the rectangle
  dest.h = h;                                                                                       // height of the rectangle

  SDL_FillRect (screen, &dest, color);                                                              // draw the black block to the screen
                                                                                                    // the white block is a bit bigger so it looks like a white frame
                                                                                                    // around the black one

}



void highscore (unsigned int reachedscore)                                                          // checks if the score is in the top-5 and enters it in the list
{

  FILE *scorelist;                                                                                  // filepointer to open highscore list
  int i, nr, entry = 0, newpos = -1;
  // i = counter variable
  // nr = number of entries in the list
  // newpos = position of the new entry in the highscore list
  highscoreinfo scoresfromlist[MAXSCORES], newscore;                                                // array for the scores in the list and for the new one
  
  
  if ((scorelist = fopen (".scores", "r + b")))                                                     // try to open highscorelist (binary) for reading
  {

    nr = fread (scoresfromlist, sizeof (highscoreinfo), MAXSCORES, scorelist);                      // read scores from the list and save number of scores in "nr"

    rewind (scorelist);                                                                             // set read/write pointer to the beginning of the file
   
    
    for (i = 0; i < MAXSCORES && !entry; i++)                                                       // check if the reached score is in the top-5
    {
            
      if (reachedscore > scoresfromlist[i].score)                                                   // score is in the top-5
      {
        
        getname (newscore.name);                                                                    // input of the name using SDL-events
              
        newscore.score = reachedscore;                                                              // copy the reached score to the structure
                                   
        entryscore (i, nr, newscore, scoresfromlist);                                               // enter the score at position "i" in the highscore list
        
        newpos = i;                                                                                 // save position of the new score to highlight it in the list
        
        if (nr < MAXSCORES)                                                                         // there are less than 5 scores in the list
          nr++;                                                                                     // new score was inserted in the middle of the list
        
        fwrite (scoresfromlist, sizeof (highscoreinfo), nr, scorelist);                             // writes the scores into the list again
        
        entry = 1;                                                                                  // score was entried into the list
      
      }       
          
        else if (i == nr && nr != MAXSCORES)                                                        // the new score is appended at the end of the list
        {
          
          getname (newscore.name);                                                                  // input of the name using SDL-events
                
          newscore.score = reachedscore;                                                            // copy the reached score to the structure
        
          scorecpy (&scoresfromlist[nr], newscore);                                                 // copies the name and the score to the structure
          
          newpos = i;                                                                               // save position of the new score to highlight it in the list
        
          nr++;                                                                                     // new score was appended, number of scores increased
          
          fwrite (scoresfromlist, sizeof (highscoreinfo), nr, scorelist);                           // writes the scores into the list again
          
          entry = 1;                                                                                // score was entried into the list
                      
        }
    
    }
      
    fclose (scorelist);                                                                             // close the highscorelist
      
    draw_highscorelist (newpos);                                                                    // draws the highscorelist onto the screen
    
  }  
  
    else if ((scorelist = fopen (".scores", "wb")))                                                 // there is no list, so it is created
    {  
      
      getname (newscore.name);                                                                      // input of the name using SDL-events
      
      newscore.score = reachedscore;                                                                // copy the reached score to the structure
      
      nr = 1;                                                                                       // the new score is the only score in the list
      
      newpos = 0;                                                                                   // it is the first (position 0) score in the list
    
      fwrite (&newscore, sizeof (highscoreinfo), 1, scorelist);                                     // writes the score into the list
      
      fclose (scorelist);                                                                           // closes the highscorelist
      
      draw_highscorelist (newpos);                                                                  // draws the highscorelist onto the screen
    
    }
 
      else                                                                                          // write into the error log if the file wasn't opened correctly
          logerror (ERROR_FILE);
        
}



void entryscore (int pos, int nr, highscoreinfo newscore, highscoreinfo list[])                     // enters the new score at the correct position in the list
{

  int i, entry = 0;                                                                                 // "i" = counter variable, "entry" = says if score is entered in the list

  for (i = nr - 1; !entry; i--)                                                                     // read the list from the lowest score to the highest
  {
  
    if (i != MAXSCORES - 1)                                                                         // if it is not the last score
      scorecpy (&list[i + 1], list[i]);                                                             // the scores are copied on their forerunner
   
    if (i == pos)                                                                                   // stop the loop at the new scores position
      entry = 1;  
  
  }
  
  scorecpy (&list[pos], newscore);                                                                  // enter the score at the correct position

}



void scorecpy (highscoreinfo *dest, highscoreinfo src)                                              // copies a "highscoreinfo" structure
{
  
  dest -> score = src.score;                                                                        // copies the score
  
  strcpy (dest -> name, src.name);                                                                  // copies the name

}



void draw_highscorelist (int newpos)                                                                // draws the highscorelist on the screen
{

  FILE *scorelist;                                                                                  // file pointer to open highscorelist
  scoresurfaces *scores;                                                                            // surfaces for the text
  SDL_Color fontcolor = {255, 255, 255}, newcolor = {0, 255, 0};                                    // normal fontcolor is white; the new entry is green
  SDL_Rect dest;                                                                                    // surface-destination
  SDL_Event event;                                                                                  // structure to store SDL-events
  TTF_Font *font;                                                                                   // font for the text
  int i, nr = MAXSCORES;                                                                            // "i" = counter variable, "nr" = number of scores
  int wait;                                                                                         // variable to wait for input
  char charhighscores[] = "HIGSCORES";                                                              // title of the window
  highscoreinfo *scoresfromlist;                                                                    // array to read the the scores from the file
  charscores *charscore;                                                                            // strings for the scores
   
  dest.y = 150;                                                                                     // starting coordinate of the highscorelist
   
  
  if ((scorelist = fopen (".scores", "rb")))                                                        // checks if file is opened correctly
  {  

    fseek (scorelist, 0, SEEK_END);                                                                 // set pointer to the end of the file
    
    nr = (ftell (scorelist) / sizeof (highscoreinfo));                                              // get number of scores in the file
    
    fseek (scorelist, 0, SEEK_SET);                                                                 // set pointer to the beginning of the file
    

    scoresfromlist = (highscoreinfo *) malloc (nr * sizeof (highscoreinfo));                        // allocate memory for the scores from the highscorelist

    nr = fread (scoresfromlist, sizeof (highscoreinfo), MAXSCORES, scorelist);                      // reads the highscores from the list
  
    fclose (scorelist);                                                                             // close the highscore list  
   
    charscore = (charscores *) malloc (sizeof (charscores) * nr);                                   // allocate memory for "nr" charscores
 
    scores = (scoresurfaces *) malloc (nr * sizeof (scoresurfaces));                                 // allocate memory for the surfaces
  
  
    for (i = 0; i < nr; i++)                                                                        // assign positions and and values for all surfaces
    {       
     
      sprintf (charscore[i].pos, "%d.", i + 1);                                                     // write ranking into the string
     
      strcpy (charscore[i].name, scoresfromlist[i].name);                                           // copy name into the string
     
      sprintf (charscore[i].score, "%u", scoresfromlist[i].score);                                  // write reached score into that string
  
    }


    font = TTF_OpenFont ("arial.ttf", 30);                                                          // open 30 pt arial font 

  
    if (font != NULL)                                                                               // check if font was opened correctly
    {
    
      draw_title (charhighscores);                                                                  // draws "HIGHSCORES" onto the screen
      
      for (i = 0; i < nr; i++)                                                                      // go through all highscores
      {
      
        if (i == newpos)                                                                            // if this is the new entry
        {
  
          scores[i].pos = TTF_RenderText_Solid (font, charscore[i].pos, newcolor);                  // the fontcolor is green
  
          scores[i].name = TTF_RenderText_Solid (font, charscore[i].name, newcolor);
  
          scores[i].score = TTF_RenderText_Solid (font, charscore[i].score, newcolor);
  
        }
        
          else                                                                                      // otherwise it is white
          {       
  
            scores[i].pos = TTF_RenderText_Solid (font, charscore[i].pos, fontcolor);                       
  
            scores[i].name = TTF_RenderText_Solid (font, charscore[i].name, fontcolor);
  
            scores[i].score = TTF_RenderText_Solid (font, charscore[i].score, fontcolor);
  
          }


        if (scores[i].pos && scores[i].name && scores[i].score)                                     // check if rendered correctly
        {
       
          dest.x = screen -> w / 10;                                                                // position of the ranking
          dest.y += 50; 
          dest.w = scores[i].pos -> w;                                                              // width of the ranking
          dest.h = scores[i].pos -> h;                                                              // height of the ranking
       
          SDL_BlitSurface (scores[i].pos, NULL, screen, &dest);                                     // blit surface onto the screen                  
       
          
          dest.x += 80;                                                                             // name is 80 pixels right of the ranking
          dest.w = scores[i].name -> w;                                                             // width of the name
          dest.h = scores[i].name -> h;                                                             // height of the name
       
          SDL_BlitSurface (scores[i].name, NULL, screen, &dest);                                    // blit surface onto the screen     
         
          
          dest.x = screen -> w - scores[i].score -> w - 60;                                         // right edge of the score is 60 pixels away of the right border
          dest.w = scores[i].score -> w;                                                            // width of the score
          dest.h = scores[i].score -> h;                                                            // height of the score
         
          SDL_BlitSurface (scores[i].score, NULL, screen, &dest);                                   // blit the surface onto the screen
          
       
          SDL_FreeSurface (scores[i].pos);                                                          // free the surface with the player's ranking
          SDL_FreeSurface (scores[i].name);                                                         // free the surface with the player's name
          SDL_FreeSurface (scores[i].score);                                                        // free the surface with the reached score
       
        }

          else                                                                                      // write into the error log if the surfaces weren't rendered correctly  
            logerror (ERROR_TTF);

      }  
      
      SDL_Flip (screen);                                                                            // flip surface (doublebuffer)                
    
      TTF_CloseFont (font);                                                                         // close the font  
    
      while (SDL_PollEvent (&event) != 0);                                                          // get all earlier events from the event queue
  
  
      wait = 1;                                                                                     // wait for input
  
      while (SDL_WaitEvent (&event) != 0 && wait)                                                   // run this loop until an event is found
      {
    
        switch (event.type)                                                                         // ask for event type
        {
      
          case SDL_KEYDOWN: wait = 0;                                                               // key was pressed, stop the loop
                            break;
      
        }
    
      }    
    
          
    }     
  
      else                                                                                          // write into the error log if the fonts weren't opened correctly
        logerror (ERROR_TTF);

    free (charscore);                                                                               // free the array of charscores  
    free (scores);                                                                                  // free the array of surfaces    
    free (scoresfromlist);                                                                          // free the allocated memory
  
  }
  
    else                                                                                            // write into the error log if file couldn't be opened correctly
      logerror (ERROR_FILE);
      
}



void draw_title (char text[])                                                                       // draws a title onto the screen
{

  Uint16 black;                                                                                     // black color
  SDL_Color fontcolor = {255, 255, 255};                                                            // fontcolor is white
  TTF_Font *font;                                                                                   // font to render text
  SDL_Surface *title;                                                                               // surface to blit "HIGHSCORE" onto the screen 
  SDL_Rect dest;                                                                                    // destination of the surface
  
  
  black = SDL_MapRGB (screen -> format, 0, 0, 0);                                                   // create a black color
  
  SDL_FillRect (screen, NULL, black);                                                               // the screen turns completely black  

 
  font = TTF_OpenFont ("arial.ttf", 40);                                                            // open 40 pt arial font

    
  if (font != NULL)                                                                                 // check if font opened co  rrectly
  {

    title = TTF_RenderText_Solid (font, text, fontcolor);                                           // render white text

    if (title != NULL)                                                                              // surface was rendered successfull
    {
    
      dest.x = (screen -> w / 2) - (title -> w / 2);                                                // x-coordinates of the title
      dest.y = 0;                                                                                   // is on top of the screen/window
      dest.w = title -> w;                                                                          // width of the title
      dest.h = title -> h;                                                                          // hight of the title

      SDL_BlitSurface (title, NULL, screen, &dest);                                                 // blit surface onto the screen

    }

      else                                                                                          // write into the error log if the surfaces weren't rendered correctly  
        logerror (ERROR_TTF);

    TTF_CloseFont (font);                                                                           // close the font

    SDL_FreeSurface (title);

  }

    else                                                                                            // write into the error log if the fonts weren't opened correctly
      logerror (ERROR_TTF);
      
}



void getname (char name[])                                                                          // input of a name using SDL-events
{

  int i = 0, stop = 0;                                                                              // "i" = counter variable, "stop" = variable to signalise to stop input
  char charkey;                                                                                     // variable to store char-value of the input
  SDL_Event event;                                                                                  // event variable to store events

  draw_nametitle ();                                                                                // draws title onto the screen
  
  SDL_Flip (screen);                                                                                // flips the surface (doublebuffer)

  while (SDL_PollEvent (&event) != 0);                                                              // get all earlier events from the event queue

  SDL_EnableKeyRepeat (400, 30);                                                                    // enable key repeat for typing

  while (!stop)                                                                                     // get input until enter was pressed
  {
  
    SDL_WaitEvent (&event);                                                                         // wait for an event
         
    switch (event.type)                                                                             // ask for the type
    {

      case SDL_KEYDOWN:                                                                             // a key was pressed
                          
                        charkey = sdltoascii (event);                                               // cast the SDL-code to ASCII
                          
                        if (charkey != 0)                                                           // a valid character was pressed
                        {
                          
                          if (isprint (charkey) != 0 && i < 15)                                     // it is a printable character and there are less than 15 characters
                          {
                       
                            name[i] = charkey;                                                      // character is appended at the end of the string
                           
                            name[i + 1] = '\0';                                                     // the next character is the end of the string
                          
                            draw_nametitle ();                                                      // draw title
                          
                            draw_input (name);                                                      // and the input onto the screen
                              
                            i++;                                                                    // next character
                       
                          }
                        
                            else if (charkey == 8 && i > 0)                                         // backspace was pressed and the string is containing characters
                            {
                          
                              name[i - 1] = '\0';                                                   // the forerunner is the end of the string
                              
                              i--;                                                                  // go back to the input of the last character
                              
                              draw_nametitle ();                                                    // draw title
                            
                              if (strlen (name) > 0)                                                // draw the input, if the string is containing characters
                                draw_input (name);
                              
                                else                                                                // otherwise just flip the screen
                                  SDL_Flip (screen);      
                          
                            }
                            
                              else if (charkey == '\n' && i > 0)                                    // enter was pressed
                                stop = 1;                                                           // the input is stopped
                        
                        }
                               
                        break;

    }


  }
  
  SDL_EnableKeyRepeat (0, 30);                                                                      // disable key repeat
  
}



char sdltoascii (SDL_Event event)                                                                   // converts a SDL event to an ASCII character
{

  char asciikey = 0;                                                                                // variable to store ascii key; -1 for a non-valid key
  SDL_keysym key = event.key.keysym;                                                                // store key informations in the structure
    
  if ((key.sym >= 97 && key.sym <= 122) && (KMOD_SHIFT & key.mod))                                  // a letter and shift was pressed
    asciikey = toupper (key.sym);                                                                   // the character is make upper case 

    else if (key.sym == SDLK_q && ((SDLK_MODE & key.mod) || (SDLK_RALT & key.mod)))                 // 'strg+alt' and 'q' were pressed
      asciikey = '@';                                                                               // the character is '@'
      
      else if (key.sym >= 97 && key.sym <= 122)                                                     // a letter was pressed
        asciikey = key.sym;                                                                         // the code must not be converted
        
        else if (key.sym >= 256 && key.sym <= 265)                                                  // a number at the keypad was pressed
          asciikey = key.sym - 208;                                                                 // it is converted to a normal number
          
          else if (key.sym >= 48 && key.sym <= 57)                                                  // a number was pressed
            asciikey = key.sym;                                                                     // the code must not be converted
            
            else if (key.sym == 8)                                                                  // backspace was pressed
              asciikey = 8;                                                                         // the code must not be converted
              
              else if (key.sym == ' ')                                                              // space was pressed
                asciikey = ' ';                                                                     // the code must not be converted
              
                else if (key.sym == SDLK_KP_ENTER || key.sym == SDLK_RETURN)                        // enter was pressed
                  asciikey = '\n';                                                                  // it is converted to ascii-enter

                  else if (key.sym >= 35 && key.sym <= 47)                                          // special characters like '.', ',', '-', etc
                    asciikey = key.sym;                                                             // the code must not be converted

                    else if (key.sym == SDLK_CARET)                                                 // '^' was pressed
                      asciikey = '^';                                                               // it is converted to ascii

  return asciikey;                                                                                  // returns the ascii-code

}



void draw_nametitle ()                                                                              // draws the title onto the screen
{

  SDL_Surface *text;                                                                                // surface to render text
  SDL_Rect dest;                                                                                    // destination of the surface
  SDL_Color fontcolor = {255, 255, 255};                                                            // white fontcolor
  TTF_Font *font;                                                                                   // font to render text to a surface
  Uint16 black;                                                                                     // black color
  
  black = SDL_MapRGB (screen -> format, 0, 0, 0);                                                   // create a black color
  
  SDL_FillRect (screen, NULL, black);                                                               // the screen turns completely black   
  
 
  font = TTF_OpenFont ("arial.ttf", 45);                                                            // open 45 pt arial font

    
  if (font != NULL)                                                                                 // check if font opened co  rrectly
  {

    text = TTF_RenderText_Solid (font, "New Highscore, enter your name: ", fontcolor);              // render the text                  
    
    if (text != NULL)                                                                               // text rendered successfully
    {
    
      dest.x = (screen -> w / 2) - (text -> w / 2);                                                 // coordinates of the surface
      dest.y = (screen -> h / 2) - (text -> h * 2);
      dest.w = text -> w;                                                                           // size of the surface
      dest.h = text -> h;
                                                                 
      SDL_BlitSurface (text, NULL, screen, &dest);                                                  // blit surface onto the screen

    }

      else                                                                                          // write into the error log if the surfaces weren't rendered correctly  
        logerror (ERROR_TTF);

    TTF_CloseFont (font);                                                                           // close the font
     
    SDL_FreeSurface (text);                                                                         // free the rendered surface

  }

    else                                                                                            // write into the error log if the fonts weren't opened correctly
      logerror (ERROR_TTF);

}



void draw_input (char name[])                                                                       // draws the input onto the screen
{

  SDL_Surface *text;                                                                                // surface to render text
  SDL_Color fontcolor = {255, 255, 255};                                                            // white fontcolor
  SDL_Rect dest;                                                                                    // destination of the text
  TTF_Font *font;                                                                                   // font to render text

                                                     
  font = TTF_OpenFont ("arial.ttf", 35);                                                            // open 35 pt arial font

    
  if (font != NULL)                                                                                 // check if font opened co  rrectly
  {

    text = TTF_RenderText_Solid (font, name, fontcolor);                                            // render the text

    if (text != NULL)                                                                               // check if rendered successfully
    {
    
      dest.x = (screen -> w / 2) - (text -> w / 2);                                                 // coordinates of the surface
      dest.y = (screen -> h / 2) - (text -> h);
      dest.w = text -> w;                                                                           // size of the surface
      dest.h = text -> h;
                                                                                
      SDL_BlitSurface (text, NULL, screen, &dest);                                                  // blit surface onto the screen

    }

      else                                                                                          // write into the error log if the surfaces weren't rendered correctly  
        logerror (ERROR_TTF);
 
    TTF_CloseFont (font);                                                                           // close the font
    SDL_FreeSurface (text);                                                                         // free the rendered surface
       
    SDL_Flip (screen);                                                                              // flip the screen (doublebuffer)

  }

    else                                                                                            // write into the error log if the fonts weren't opened correctly
      logerror (ERROR_TTF);
  
}



int yesorno (char text[])                                                                           // asks the player a question
{

  int decision = 1, first = 1; 
  // "decision" = current position of the cursor (on "yes" or "no")
  // "first" = the user hasn't pressed a button so far
  Uint16 black;                                                                                     // black color
  SDL_Surface *yes, *no, *again;                                                                    // surfaces to render text
  SDL_keysym key;                                                                                   // used to store event, if a key was pressed
  SDL_Color fontcolor = {255, 255, 255}, chosen = {0, 255, 0};                                      // white and green color
  SDL_Rect destagain, destyes, destno;                                                              // destinations of the surfaces
  SDL_Event event;                                                                                  // variable to store an event
  TTF_Font *font, *bigfont;                                                                         // two fonts; big one for question and normal one for "yes" and "no"

  while (SDL_PollEvent (&event) != 0);                                                              // get all earlier events from the event queue

  while (1)                                                                                         // run this loop until the user pressed enter and "return" stops it
  {
    
    if (!first)                                                                                     // if it is the first frame, no events are captured
    {
    
      SDL_WaitEvent (&event);                                                                       // wait for an event
    
      switch (event.type)                                                                           // ask for event type
      {
    
        case SDL_KEYDOWN:                                                                           // a key was pressed
                           
                           key = event.key.keysym;                                                  // copy the event details to "key"
      
                           if (key.sym == SDLK_RIGHT)                                               // user pressed right-arrow
                             decision = 0;                                                          // current decision is no
                           
                             else if (key.sym == SDLK_LEFT)                                         // user pressed left-arrow
                               decision = 1;                                                        // current decision is yes
                             
                               else if (key.sym == SDLK_KP_ENTER || key.sym == SDLK_RETURN)         // user pressed enter
                                 return decision;                                                   // return the decision
    
      }  
      
    } 
    
    first = 0;                                                                                      // it is not the first frame anymore

    black = SDL_MapRGB (screen -> format, 0, 0, 0);                                                 // create a black color
    SDL_FillRect (screen, NULL, black);                                                             // the screen turns completely black    
 
                                                     
    bigfont = TTF_OpenFont ("arial.ttf", 45);                                                       // open 45 pt arial font
    font = TTF_OpenFont ("arial.ttf", 35);                                                          // open 35 pt arial font
 
    if (font != NULL && bigfont != NULL)                                                            // check if the fonts were opened correctly
    {      
  
      again = TTF_RenderText_Solid (bigfont, text, fontcolor);                                      // render the question if the player wants to play again in a big font
    
      if (decision == 1)                                                                            // current decision is "yes"
        yes = TTF_RenderText_Solid (font, "YES", chosen);                                           // it is rendered in a green font

        else                                                 
          yes = TTF_RenderText_Solid (font, "YES", fontcolor);                                      // otherwise it is rendered in a white font       

      if (decision == 0)                                                                            // current decision is "no"
        no = TTF_RenderText_Solid (font, "NO", chosen);                                             // it is rendered in a green font

        else
          no = TTF_RenderText_Solid (font, "NO", fontcolor);                                        // otherwise it is rendered in a white font


      if (again && yes && no)                                                                       // all surfaces were rendered successfully
      {
    
        destagain.x = (screen -> w / 2) - (again -> w / 2);                                         // coordinates of "again"
        destagain.y = (screen -> h / 3) - (again -> h);
        destagain.w = again -> w;                                                                   // size of "again"
        destagain.h = again -> h;
      
        SDL_BlitSurface (again, NULL, screen, &destagain);                                          // blit the surface onto the screen
      
      
        draw_blockwithframe ((screen -> w / 2) - yes -> w - 10 - 2,                                 // a white frame is created
                             (screen -> h / 2) - yes -> h - 2, yes -> w + 4, yes -> h + 4);
      
        destyes.x = (screen -> w / 2) - yes -> w - 10;                                              // position of "yes"
        destyes.y = (screen -> h / 2) - yes -> h;
        destyes.w = yes -> w;                                                                       // size of "yes"
        destyes.h = yes -> h;      
      
        SDL_BlitSurface (yes, NULL, screen, &destyes);                                              // blit the surface onto the screen


        draw_blockwithframe ((screen -> w / 2) + 10 - 2,                                            // a white frame is created
                             (screen -> h / 2) - yes -> h - 2, yes -> w + 4, yes -> h + 4);

        destno.x = (screen -> w / 2) + 10;                                                          // coordinates of "no"
        destno.y = (screen -> h / 2) - no -> h;
        destno.w = no -> w;                                                                         // size of "no"
        destno.h = no -> h;         
      
        SDL_BlitSurface (no, NULL, screen, &destno);                                                // blit surface onto the screen
               
      
        SDL_Flip (screen);                                                                          // flip the screen (doublebuffer)
      
        SDL_FreeSurface (again);                                                                    // free the rendered surfaces
        SDL_FreeSurface (yes);
        SDL_FreeSurface (no);
      
      }

        else                                                                                          // write into the error log if the surfaces weren't rendered correctly  
          logerror (ERROR_TTF);
        
      TTF_CloseFont (font);                                                                         // close the fonts
      TTF_CloseFont (bigfont);
   
    }
  
      else                                                                                          // write into the error log if the fonts weren't opened correctly
        logerror (ERROR_TTF);

  }

}



void deletesnake ()                                                                                 // frees the allocated memory
{

  block_info *delhelp;                                                                              // help pointer to delete list
  
  while (snake != NULL)                                                                             // free memory until no block is left
  {
  
    delhelp = snake -> next;                                                                        // save position of next block
    
    free (snake);                                                                                   // free block
    
    snake = delhelp;                                                                                // go to next element
    
    
    delhelp = oldpos -> next;                                                                       // save position of next block
    
    free (oldpos);                                                                                  // free block
    
    oldpos = delhelp;                                                                               // go to next element
  
  }
  
  snake = NULL;                                                                                     // both lists are empty
  oldpos = NULL;

}



void removelogerror ()                                                                              // deletes the error log, to create a new one
{

  remove ("error.log");                                                                             // remove the file

}



void logerror (int type)                                                                            // write error messages into a file
{

  FILE *errorlog;                                                                                   // filepointer to the error log                     
  
  errorlog = fopen ("error.log", "a");                                                              // open error log, appendig error
  
  switch (type)                                                                                     // ask for type of error
  {
  
    case ERROR_SDL: fprintf (errorlog, "SDL-Error: %s\n", SDL_GetError ());                         // SDL-error
            break;
            
    case ERROR_TTF: fprintf (errorlog, "TTF-Error: %s\n", TTF_GetError ());                         // TTF-error
            break;
            
    case ERROR_FILE: fprintf (errorlog, "Unable to open file: %s\n", strerror (errno));             // file-error
            break;               
            
  }
  
  fclose (errorlog);                                                                                // close file

}
